pro dirbe_read,file=file,fields=fields,dt1,dt2,dt3,dt4,dt5,dt6,$
       dt7,dt8,dt9,dt10,dt11,dt12,dt13,dt14,dt15,$
       startpixel=startpixel,stoppixel=stoppixel

;+NAME/ONE LINE DESCRIPTION OF ROUTINE:
;    DIRBE_READ provides user-friendly access to DIRBE FITS maps.
;
;DESCRIPTION:
;    This IDL procedure reads in the various fields of the
;    DIRBE Project Data Set FITS files. This is a shell around
;    a number of linkimage'd C routines. If not supplied on the 
;    command line, the user is prompted for the FITS data file 
;    name (including directory), and/or if missing a list of
;    fields is displayed to choose from. The output from this
;    procedure is the specified data lists.
;
;CALLING SEQUENCE:
;    dirbe_read,[file=file],[fields=fields],dt1,[dt2,dt3,dt4...dt15],
;               [,startpixel=startpixel] [,stoppixel=stoppixel]
;
;ARGUMENTS (I = input, O = output, [] = optional):
;    [file]          I     str                 Skymap file
;    [fields]        I     str                 Skymap data fields
;    dt1,[dt2...dtn] O     int/flt/dbl/cpx arr Returned data list(s)
;    [startpixel]    I     long                Valid only for DCAF
;    [stoppixel]     I     long                Valid only for DCAF
;
;WARNINGS:
; 1)   The DIRBE PDS's may be very large and therefore use a
;      significant amount of memory in IDL. The user should take 
;      care in choosing which and how many fields to read in.
;
; 2)   This IDL procedure calls a C program which calls a FORTRAN
;      subroutine. On some systems this sequence causes problems
;      with pathways. When reading a file if it can not find your
;      file with the specified path, you may have to change
;      directories to the one in which the file is located and then
;      simply read the filename, and then change back to your work
;      directory. This can all be done within your IDL session.
;
;EXAMPLE: 
;
; To read in the photometry and standard deviation for the DIRBE
; week 10 weekly averaged maps:
;     IDL> file='path/dirbe_wk10_weekly_averaged_skymap.fits'
;     IDL> fields='photomet,stddev'
;     IDL> dirbe_read,file=file,fields=fields,phot,stddev
;
; The output arrays, 'phot,stddev' will both have dimensions (10,208283).
;
;#
;COMMON BLOCKS:
;    None
;
;PROCEDURE (AND OTHER PROGRAMMING NOTES):
;    Gets FITS file name and check if file is proper format.
;    Obtain the number of fields specified and build proper data string.
;    Open FITS file and discern which DIRBE product it is.
;    Call C functions to read data.
;    Return data into specified arrays.
;
;PERTINENT ALGORITHMS, LIBRARY CALLS, ETC.:
;    routines called: is_fits,headfits,sxpar
;    C functions called: dirbe_aaf,dirbe_dcaf,dirbe_e90,dirbe_gpm,
;                        dirbe_wk
;
;MODIFICATION HISTORY
;    Written by J.S. Newmark, Applied Research Corp.   Sep 1995
;
;.TITLE
;Routine DIRBE_READ
;-
;
IF N_params() LT 1 THEN BEGIN
   print,'Syntax - dirbe_read,file=file,fields=fields,dt1,[dt2,dt3,dt4...dt15]'
   print,'               [,startpixel=startpixel] [,stoppixel=stoppixel]'
   stat = 0
   GOTO, abort
ENDIF
IF NOT KEYWORD_SET(file) THEN BEGIN
   file = ''
   read,'Enter file name: ',file
ENDIF
stat=is_fits(file)
IF stat NE 1 THEN BEGIN
   PRINT,'Specified file is not a FITS file'
   GOTO, abort
ENDIF

n_flds = 1
IF (N_ELEMENTS(fields) NE 0) THEN BEGIN
     FOR i=0,STRLEN(fields)-1 DO $
       	 IF (STRMID(fields,i,1) EQ ',') THEN n_flds = n_flds + 1 
ENDIF ELSE fields=''

data_str = ''
dat_str = STRARR(8)
FOR i=0,n_flds-1 DO BEGIN
	dat_str(i) = STRCOMPRESS('dt' + STRING(i+1),/remove_all)
	data_str = data_str + ',' + dat_str(i)
ENDFOR

IF KEYWORD_SET(startpixel) THEN $
    data_str=data_str+',startpixel='+strtrim(startpixel,2)
IF KEYWORD_SET(stoppixel) THEN $
    data_str=data_str+',stoppixel='+strtrim(stoppixel,2)

header =headfits(file)
product = sxpar(header, 'PRODUCT')
plen=strlen(strtrim(product))
print,'Reading product= ',product
IF strtrim(product) EQ 'GPM' THEN $
      status = EXECUTE('stat = dirbe_gpm(file,fields' + data_str + ')')
IF strtrim(product) EQ 'DCAF' THEN $
      status = EXECUTE('stat = dirbe_dcaf(file,fields' + data_str + ')')
IF strmid(product,plen-3,3) EQ 'AAM' THEN $
      status = EXECUTE('stat = dirbe_aaf(file,fields' + data_str + ')')
IF strmid(product,plen-3,3) EQ 'E90' THEN $
      status = EXECUTE('stat = dirbe_e90(file,fields' + data_str + ')')
IF strmid(product,0,7) EQ 'WEEKMAP' THEN $
      status = EXECUTE('stat = dirbe_wk(file,fields' + data_str + ')')

abort:
IF stat NE 1 THEN PRINT,'Program failed' 

end
